"""Streamer

Response streaming

"""

from typing import Tuple, Union

from luna3.common.luna_response import LunaResponse

from ..errors.errors import ErrorInfo
from ..web.request import VLRequest
from ..web.utils import decodeBytes, loadJson

GENERATOR_CHUNK_SIZE = 1024**3


async def streamResponse(
    response: LunaResponse, request: VLRequest, headers: dict, successStatus: int = 200
) -> Tuple[int, Union[ErrorInfo, None]]:
    """
    Stream response to request
    Args:
        response: luna response object to get data from
        headers: headers to respond with
        request: request to respond to
        successStatus: status indicating successful request
    Returns:
        response stream chunks
    """
    if response.rawResponse.status != successStatus:
        errorBody = await response.rawResponse.read()
        errorInfo = loadJson(decodeBytes(body=errorBody))
        return response.rawResponse.status, ErrorInfo.fromDict(errorInfo)
    resp = await request.respond(
        content_type=response.contentType,
        headers=headers,
    )
    async for chunk in response.iter():
        await resp.send(chunk)
    await resp.eof()
    return successStatus, None
